@@ -2,9 +2,9 @@
 #
 # Part of the Rattle package for Data Mining
 #
-# Time-stamp: <2008-06-21 14:49:36 Graham Williams>
+# Time-stamp: <2009-01-05 10:33:49 Graham Williams>
 #
-# Copyright (c) 2008 Togaware Pty Ltd
+# Copyright (c) 2009 Togaware Pty Ltd
 #
 # This files is part of the Rattle suite for Data Mining in R.
 #
@@ -27,7 +27,9 @@ pmml.kmeans <- function(model,
                         model.name="KMeans_Model",
                         app.name="Rattle/PMML",
                         description="KMeans cluster model",
-                        copyright=NULL, ...)
+                        copyright=NULL,
+                        transforms=NULL,
+                        ...)
 {
   require(XML, quietly=TRUE)
   
@@ -37,6 +39,10 @@ pmml.kmeans <- function(model,
 
   field <- NULL
   field$name <-  colnames(model$centers)
+  
+  orig.fields <- field$name
+  if (exists("pmml.transforms") && ! is.null(transforms))
+    field$name <- unifyTransforms(field$name, transforms)
   number.of.fields <- length(field$name)
 
   field$class <- rep("numeric", number.of.fields) # All fields are numeric
@@ -59,7 +65,7 @@ pmml.kmeans <- function(model,
 
   # PMML -> ClusteringModel
 
-  cl.model <- xmlNode("ClusteringModel",
+  the.model <- xmlNode("ClusteringModel",
                       attrs=c(modelName=model.name,
                         functionName="clustering", # Required
                         algorithmName="KMeans: Hartigan and Wong",
@@ -68,21 +74,36 @@ pmml.kmeans <- function(model,
 
   # PMML -> ClusteringModel -> MiningSchema
 
-  cl.model <- append.XMLNode(cl.model, pmmlMiningSchema(field))
+  the.model <- append.XMLNode(the.model, pmmlMiningSchema(field))
 
+  # PMML -> ClusteringModel -> LocalTransformations -> DerivedField -> NormContiuous
+
+  if (exists("pmml.transforms") && ! is.null(transforms))
+    the.model <- append.XMLNode(the.model, pmml.transforms(transforms))
+  
   # PMML -> ClusteringModel -> ComparisonMeasure
   
-  cl.model <- append.XMLNode(cl.model,
+  the.model <- append.XMLNode(the.model,
                              append.XMLNode(xmlNode("ComparisonMeasure",
                                                     attrs=c(kind="distance")),
                                             xmlNode("squaredEuclidean")))
+
+  # PMML -> ClusteringField
+
+  for (i in orig.fields)
+  {
+    the.model <- append.xmlNode(the.model,
+                               xmlNode("ClusteringField",
+                                       attrs=c(field=i,
+                                         compareFunction="absDiff")))
+  }
   
   # PMML -> ClusteringModel -> Cluster -> Array
   
   clusters <- list()
   for (i in 1:number.of.clusters)
   {
-    cl.model <- append.XMLNode(cl.model,
+    the.model <- append.XMLNode(the.model,
                                xmlNode("Cluster",
                                        attrs=c(name=cluster.names[i],
                                          size=model$size[i]),
@@ -92,7 +113,7 @@ pmml.kmeans <- function(model,
                                                paste(model$centers[i,],
                                                      collapse=" "))))
   }
-  pmml <- append.XMLNode(pmml, cl.model)
+  pmml <- append.XMLNode(pmml, the.model)
 
   return(pmml)
 }
